$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$psDataFolder   = $scriptPathParent + "\PSData"; 
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Append Generates Quotes".PadRight(28);
$culture = New-Object System.Globalization.CultureInfo("en-US"); # Getting culture for proper formatting of number

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output

$psDataFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<PSDataFolder>"))+1)).Replace("</PSDataFolder>","");
if ($psDataFolderCfg -ne $null -and $psDataFolderCfg -ne "") {$psDataFolder = $psDataFolderCfg; $str="PS Data root folder changed to $psDataFolder";} 
else {$str="Config <PSDataFolder> not specified, so using default location"; $psDataFolderCfg = "";}
if (!(Test-Path $psDataFolder)) {New-Item $psDataFolder -type directory}; # Create Data folder
$str | Out-File $logFile -Encoding OEM -Append;

# Getting Min Date from configuration file
$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>",""); # Getting minimum date for quote extract
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {  # Checking if minimum date makes sense
   $str="*** Error. Minimum date if file $minDateFile should be between 1960-01-01 and 2050-01-01. Fix and restart"; $str; $str | Out-File $logFile -Encoding OEM -Append; exit(1);
} 

$colSepTxt = ($config | Select-Object -Index(($config.IndexOf("<ColumnSeparator>"))+1)).Replace("</ColumnSeparator>",""); $colSep = "`t";
if ($colSepTxt.ToLower() -eq "comma") {$colSep = ","}; if ($colSepTxt.ToLower() -eq "tab") {$colSep = "`t"}; if ($colSepTxt.ToLower() -eq "verticalbar") {$colSep = "|"};

$decSep = "."; $decSep = ($config | Select-Object -Index(($config.IndexOf("<DecimalSeparator>"))+1)).Replace("</ColumnSeparator>",""); $decSep = $decSep.Substring(0, 1);

$includeArchiveFlag = $false;
$includeArchiveValue = ($config | Select-Object -Index(($config.IndexOf("<IncludeQuoteArchiveFolder>"))+1)).Replace("</IncludeQuoteArchiveFolder>",""); # Getting IncludeArchive value
if ($includeArchiveValue -ne $null) {if ($includeArchiveValue.Trim().ToLower() -eq "yes") {$includeArchiveFlag = $true;}} 

# #######################################################################################
# ######################## Adding Generated Quotes
# #######################################################################################
$outFile = $psDataFolder + "\Quotes.csv"; $recCount = 0; $getSymbolCount = 0;
$listStart = $config.IndexOf("<GeneratedQuotes>"); $listEnd = $config.IndexOf("</GeneratedQuotes>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {$symbolCount=0;}
else {$symbolList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); $symbolCount = $symbolList.Count;} #list of symbols we will work on
if ($symbolCount -gt 0) {
    # First create list of monthly dates that will be used for generated quotes
    $dToday = Get-Date; $dDate = [datetime]::ParseExact($minDate,"yyyy-MM-dd",$null); $strMthlyDate  = $dDate.ToString("yyyy-MM-dd"); 
    do {
        $dDate = $dDate.AddMonths(1); $dDate = $dDate.AddDays((-1) * $dDate.Day+1);
        if ($dDate.ToString("ddd",$culture) -eq "Sat") {$dDate = $dDate.AddDays(2);}; if ($dDate.ToString("ddd",$culture) -eq "Sun") {$dDate = $dDate.AddDays(1);}; 
        if ($dDate -gt [datetime]::ParseExact($minDate,"yyyy-MM-dd",$null) -and $dDate -le (Get-Date)) {$strMthlyDate+=","+$dDate.ToString("yyyy-MM-dd");}
    } While ($dDate -lt $dToday);

    # Generate quotes for each symbol one per month
    ForEach ($s in $symbolList) {
        $symbolLine = $s.Split(","); $str = ""; $getSymbolCount++;
        $symbol = $symbolLine[0]; $minSymbolDate = $symbolLine[1];$maxSymbolDate = $symbolLine[2];  $price = $symbolLine[3];

        if ($minSymbolDate.Trim() -eq "" -or $minSymbolDate -lt $minDate) {$minSymbolDate = $minDate;}
        if ($maxSymbolDate.Trim() -eq "" -or $maxSymbolDate -gt (Get-Date).ToString("yyyy-MM-dd")) {$maxSymbolDate = (Get-Date).ToString("yyyy-MM-dd");} 
        ForEach($d in $strMthlyDate.Split(",")) {
            if($d -ge $minSymbolDate -and $d -le $maxSymbolDate) {$str+=$d + "," + $price + "," + $symbol + "`r`n"; $recCount++;}
        } 
        if ($str.Length -ge 2) {$str = $str.Substring(0, $str.Length-2);}
        if ($str -eq "") {$str = $minSymbolDate + "," + $price + "," + $symbol; $recCount++;}
        if ($colSep -ne ",") {$str = $str.Replace(",", $colSep)}; if ($decSep -ne ".") {$str = $str.Replace(".", $decSep)};
        $str | Out-file $outFile -Encoding OEM -Append;
    }
    $str=(Get-Date).ToString("HH:mm:ss") + " Finished appending to file   Quotes.csv. Generated quote symbol count: " + $getSymbolCount.ToString().PadLeft(3) + ". Record count: $recCount.";
    $str | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$str};
}
# #######################################################################################
$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
$str=(Get-Date).ToString("HH:mm:ss") + " --- Done. Duration: $duration`r`n"
$str+="=======================================================================================`r`n";
$str | Out-File $logFile -Encoding OEM -Append; 
if ($verbose) {$str} else {$logSummary + ". Duration: $duration";};

