# 2016-March-10. Created by Maxim T. 
$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$dataRootFolder = $scriptPathParent; 
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Exchange Rates BoC".PadRight(28);

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output
$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>","");
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {$str="*** Error. Min date in psconfig file is $minDateFile but should be between 1960-01-01 and 2050-01-01"; $str | Out-File $logFile -Encoding OEM -Append; $str; exit(1);}
$dataRootFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<DataRootFolder>"))+1)).Replace("</DataRootFolder>","");
if ($dataRootFolderCfg -ne $null -and $dataRootFolderCfg -ne "") {$dataRootFolder = $dataRootFolderCfg}; "Data root folder: $dataRootFolder" | Out-File $logFile -Encoding OEM -Append;
if (!(Test-Path -Path $dataRootFolder)) {New-Item $dataRootFolder -type directory};

$quotesFolder = $dataRootFolder + "\CurrExch\"; 
if (!(Test-Path $quotesFolder)) {New-Item $quotesFolder -type directory}; # Create folder
$quoteIDFileYA = $dataRootFolder + "\QuotesIntraDay\YahooCurrExchIntraday.txt"; if (Test-Path $quoteIDFileYA) { Remove-Item $quoteIDFileYA;}

$listStart = $config.IndexOf("<Currency>"); $listEnd = $config.IndexOf("</Currency>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {$str="<currency> Symbol list is empty. Exiting script.";if($verbose){$str}; $str | Out-File $logFile -Encoding OEM -Append; exit(1);}
$currList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); #list of symbols we will work on
$currCount= $currList.Count;
$str= "=======================================================================================`r`n";
$str+=(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name + ". Symbol count: " + $currCount + ". MinDate: $minDate";
if($verbose) {$str}; $str | Out-File $logFile -Encoding OEM -Append;
$culture = New-Object System.Globalization.CultureInfo("en-US"); # Getting culture for proper formatting of number

#http://www.bankofcanada.ca/valet/observations/FXUSDCAD/csv?start_date=2017-04-28&end_date=2017-05-03
$cur = @("AED","ARS","AUD","BRL","CAD","CHF","CLP","CNY","DKK","EUR","GBP","HKD","IDR","ILS","INR","JPY","KRW","MXN","MYR","NOK","NZD","SEK","SGD","TWD","USD","ZAR");
# If From or To currency is CAD then we can request conversion rate directly. Otherwise we should request rates: CurrFrom->CAD and CAD->CurrTo and then multiple these values
$urlBase1 = "http://www.bankofcanada.ca/valet/observations/FX@@CurrFrom@@@@CurrTo@@/csv?start_date=@@DateFrom@@&end_date=@@DateTo@@"
$urlBase2 = "http://www.bankofcanada.ca/valet/observations/FX@@CurrFrom@@CAD,FXCAD@@CurrTo@@/csv?start_date=@@DateFrom@@&end_date=@@DateTo@@"

$dateTo = Get-Date -Format "yyyy-MM-dd";

if ($currCount -le 1) {"Found just one currency, will not create currency exchange database"; exit(1);}
For($i=0; $i -lt $currCount;$i++) {if ($cur.IndexOf($currList[$i]) -eq -1) {"*** Error. Currency " + $currList[$i] + " is not supported."; "*** Error. Currency " + $currList[$i] + " is not supported." | Out-File $logFile -Encoding OEM -Append; exit(1)}};

For($currFromIndex=0; $currFromIndex -lt $currCount; $currFromIndex++) {
    For($currToIndex=0; $currToIndex -lt $currCount; $currToIndex++) {
        if ($currList[$currFromIndex] -ne $currList[$currToIndex]) {
            $currFrom = $currList[$currFromIndex]; $currTo = $currList[$currToIndex]; $lastDateInFile = "";
            $log = (Get-Date).ToString("HH:mm:ss") + " " + $currFrom + "->" + $currTo;

            $symbolQuoteFile = $quotesFolder + $currFrom + "_" + $currTo + ".txt";
            $nextDate = $minDate;
            # Getting date of last quote for currency group in quote file we already have. If there is no quote file, use min date from parameter file
            if (Test-Path -Path $symbolQuoteFile) { # File exists, need to find last available quote date
                $fc = @(import-csv $symbolQuoteFile -Header "Date","Rate","CurrFrom","CurrTo" | Sort-Object -Property "Date"); $lastDateInFile=""; if ($fc.count -gt 0) {$lastDateInFile = $fc[$fc.count-1].Date;}
                if ($lastDateInFile -ne "") {
                    $nextDate = ([datetime]::ParseExact($lastDateInFile,"yyyy-MM-dd",$null)).AddDays(1).ToString("yyyy-MM-dd"); # Adding one day to max date found in quote file - this will be next request start date
                    "  Found rate file. Looking for quotes starting from: $nextDate" | Out-File $logFile -Encoding OEM -Append;
                }
             }
            $log += " Last local quote: " + $lastDateInFile.PadRight(10) + ".";
            
            if (($currFrom.ToLower() -eq "cad") -or ($currTo.ToLower() -eq "cad")) {$urlBase = $urlBase1; $useCad=$false;} else {$urlBase = $urlBase2; $useCad=$true;}; # Direct conversion or using CAD in Between

            $url = $urlBase.Replace("@@CurrFrom@@",$currFrom).Replace("@@CurrTo@@",$currTo).Replace("@@DateFrom@@", $nextDate).Replace("@@DateTo@@",$dateTo);
            "  Url: " + $url | Out-File $logFile -Encoding OEM -Append;

            $wcD = new-object system.net.WebClient; $webpageD = ""; $reqCount++; $reqRows = 0; $lastQuote = ""; 
            try {$webpageD = $wcD.DownloadData($url);} # Get page from url
            catch { $reqFailed++; "  " + $symbol + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; if ($verbose) {$log + " No new data received."}; continue; } # if attempt to get webpage failed go to next symbol

            $cfD = [System.Text.Encoding]::ASCII.GetString($webpageD); # This variable now contains downloaded currency rates
            if ($cfD.Contains("<html")) {$reqFailed++; "  No new data (returned html)" | Out-File $logFile -Encoding OEM -Append; if($verbose) {$log + " No new data received."}; continue; } # Result is html file, something went wrong, ignore result that was just returned
            
            $clD = $cfD.Split("`n"); $reqSucceed++;
            if ($clD.Contains("ERRORS")) {$clD = $clD[0..$clD.IndexOf("ERRORS")];} # Just lines before "ERRORS"
             # First 9 lines just quote header; Ignore rows that says "Holiday"; Ignore empty rows; Ignore rows with quote date before $DateFrom; Ignore quotes where rate is empty value.
             ($clD | select -Skip 9| Where {$_ -notlike "*holiday*" -and $_ -like "2*"} | Where {$_ -ne ""} | Where {$_.split(",")[1].Trim() -ne ""} |
                %{$a=$_.Split(",");if ($a[0] -ge $nextDate) {$reqRows++; if(!$useCad) {$exchRate = $a[1];} else {$exchRate = ([convert]::ToDouble($a[1],$culture) * [convert]::ToDouble($a[2],$culture)).ToString($culture);}; $a[0] + "," + $exchRate + "," + $CurrFrom + "," + $CurrTo; if ($a[0] -gt $lastQuote) {$lastQuote=$a[0];}} }) | 
                Out-File $symbolQuoteFile -Encoding OEM -Append;

            "Done: $currFrom->$currTo (from $nextDate to $dateTo). New record count: $reqRows." | Out-File $logFile -Encoding OEM -Append;
            if ($reqRows -eq 0) {if($verbose) {$log + " No new data found."} }
            else { if($verbose) {$log + " New rec count " + $reqRows.ToString().PadLeft(4) + ". Last quote: $lastQuote" }};

        }
    }
}

$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
$str =(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. New records: $reqRows. Duration: $duration`r`n";
$str+="=======================================================================================`r`n";
$logSummary += ". Quotes Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. New records: $reqRows. Duration: $duration";
if($verbose) {$str} else{$logSummary;}; $str | Out-File $logFile -Encoding OEM -append;
