# 2016-March-10. Created by Maxim T. 
$startTime      = GET-DATE; $reqCount = 0; $reqFailed = 0; $reqSucceed = 0;
$scriptPath     = Split-Path -parent $MyInvocation.MyCommand.Definition; $scriptPathParent = Split-Path -path $scriptPath -Parent; 
$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); if (!(Test-Path -Path  ($scriptPath + "\Log\"))) {New-Item ($scriptPath + "\Log\") -type directory};
$configFile     = $scriptPath + "\psConfig.txt"; if (!(Test-Path -Path $configFile)) {"Config file not found: $configFile"; exit(1)};
$dataRootFolder = $scriptPathParent; 
"Starting script. " + (Get-Date).ToString("yyyy-MM-dd hh:mm:ss") | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: Yahoo Currency".PadRight(28);

$config = gc $configFile | ? {$_.trim() -ne "" -and !$_.StartsWith("#") }; # Getting config file without empty lines and comments
$verbose = $false; if (($config | Select-Object -Index(($config.IndexOf("<DetailOutput>"))+1)).Replace("</DetailOutput>","").ToLower() -eq "yes") {$verbose=$true;};  # Check if configured to do detail output
$minDate = ($config | Select-Object -Index(($config.IndexOf("<MinDate>"))+1)).Replace("</MinDate>","");
if ($minDate.Length -lt 10 -or $minDate -le "1960-01-01" -or $minDate -ge "2050-01-01") {$str="*** Error. Min date in psconfig file is $minDateFile but should be between 1960-01-01 and 2050-01-01"; $str | Out-File $logFile -Encoding OEM -Append; $str; exit(1);}
$dataRootFolderCfg = ($config | Select-Object -Index(($config.IndexOf("<DataRootFolder>"))+1)).Replace("</DataRootFolder>","");
if ($dataRootFolderCfg -ne $null -and $dataRootFolderCfg -ne "") {$dataRootFolder = $dataRootFolderCfg}; "Data root folder: $dataRootFolder" | Out-File $logFile -Encoding OEM -Append;
if (!(Test-Path -Path $dataRootFolder)) {New-Item $dataRootFolder -type directory};

$quotesFolder  = $dataRootFolder + "\CurrExch\"; if (!(Test-Path -Path $quotesFolder)) {$str="Quotes folder not found: $quotesFolder"; $str; $str | Out-File $logFile -Encoding OEM -Append; exit(1)};
$quoteIDFileYA = $dataRootFolder + "\QuotesIntraDay\YahooCurrExchIntraday.txt"; if (Test-Path $quoteIDFileYA) { Remove-Item $quoteIDFileYA;}

$listStart = $config.IndexOf("<Currency>"); $listEnd = $config.IndexOf("</Currency>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {$str="<currency> Symbol list is empty. Exiting script.";if($verbose){$str}; $str | Out-File $logFile -Encoding OEM -Append; exit(1);}
$currList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); #list of symbols we will work on
if ($currList.Count -eq 1) {"Just one currency, no need to get exchange rates"| Out-File $logFile -Encoding OEM -Append; exit(1);} 
$currCount = $currList.Count; 
$symbolList = @(); # List of currency From/To pairs in the format #CADUSD=X,CAD,USD for Yahoo requests
For($currFromIndex=0; $currFromIndex -lt $currCount; $currFromIndex++) {
    For($currToIndex=0; $currToIndex -lt $currCount; $currToIndex++) {
        if ($currList[$currFromIndex] -ne $currList[$currToIndex]) {
            $symbolList +=, ($currList[$currFromIndex]+$currList[$currToIndex] + "=X,"+$currList[$currFromIndex]+","+$currList[$currToIndex])          #CADUSD=X,CAD,USD
        }
    }
}
$str= "=======================================================================================`r`n";
$str+=(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name + ". Symbol count: " + $symbolList.count + ". MinDate: $minDate";
if($verbose) {$str}; $str | Out-File $logFile -Encoding OEM -Append;
$urlBaseStart = "http://download.finance.yahoo.com/d/quotes.csv?s="; # This is start of url for latest quote
$urlBaseEnd   = "&f=sl1d1t1&e=.csv"; # URL request parameters: s-Symbol; l1-Last trade (Price Only); d1-Last trade date; t1-Last trade time; 

$rowCount = $symbolList.Count; $QuotesFound = 0; $reqFailed=0; $newQuotes =0; $lastDateTime = "";
$yS=@(); $cF=@(); $cT=@(); $sList = "";
For($i=0; $i -lt $rowCount; $i++) {
    $line = $symbolList[$i].Split(",");
    $yS+= ,$line[0]; $cF += ,$line[1]; $cT += ,$line[2]; $sList += $line[0] + ",";
}
$sList = $sList.Substring(0, $sList.Length-1);
$url = $urlBaseStart + $sList + $urlBaseEnd; "URL: $url" | Out-File $logFile -Encoding OEM -append;  #building URL to submit
$wc = new-object system.net.WebClient; $webpage = "";
try {$webpage = $wc.DownloadData($url); }# Get page from url.
catch { $reqFailed++; "Request failed." | Out-File $logFile -Encoding OEM -append; continue;} # if attempt to get webpage failed, inform user and go to next request
$data = [System.Text.Encoding]::ASCII.GetString($webpage); # This variable now contains downloaded quotes
if ($data.Contains("<html>")) { $reqFailed++; "Request failed (return html)." | Out-File $logFile -Encoding OEM -append; continue;}
$data = $data.Replace("`"","").Split("`n");
$result = $data | ?{$_ -ne "" -and !$_.StartsWith("N/A")}; 

ForEach($r in $result) { # going through each record that was returned 
    $rSplit = $r.Split(","); $price = $rSplit[1]; $symbol = $rSplit[0]; $time = $rSplit[3];
    $date = $rSplit[2]; $date = $date.Split("/"); $date = $date[2]+ "-" + $date[0].PadLeft(2,"0") + "-" + $date[1].PadLeft(2,"0"); #Getting date and converting from m/d/yyyy to yyyy-mm-dd
    if ($symbol -eq "" -or $symbol -eq "N/A") {continue;} # wrong symbol, ignoring line
    if ($price  -eq "" -or $price  -eq "N/A" -or $price -eq "0" -or $price -eq "0.00") {"Something wrong with Price $price" | Out-File $logFile -Encoding OEM -append; continue;} # wrong last price, ignore line
    if ($date.Length -ne 10 -or $date -lt (Get-Date).AddDays(-10).ToString("yyyy-MM-dd") -or $date -gt (Get-Date).ToString("yyyy-MM-dd")) {"Something wrong with Date $date" | Out-File $logFile -Encoding OEM -append; continue;} #something wrong with date
    
    try {$dateD = ([datetime]::ParseExact($date,"yyyy-MM-dd",$null)); $dayOfWeek = $dateD.DayOfWeek.ToString();} 
    catch {$dayOfWeek = "Saturday";}
    if ($dayOfWeek -eq "Saturday" -or $dayOfWeek -eq "Sunday") {$weekendFlag = $true} else {$weekendFlag=$false};

    if ($yS.IndexOf($symbol) -eq -1) {continue;} # returned symbol not found in our list;
    $currFrom = $cF[$yS.IndexOf($symbol)]; $currTo = $cT[$yS.IndexOf($symbol)]; $lastDateInFile = "";
    $log = (Get-Date).ToString("HH:mm:ss") + " $currFrom-$currTo"; if($verbose) {"Symbol: $symbol" | Out-File $logFile -Encoding OEM -Append}; 
    $symbolQuoteFile = $quotesFolder + $currFrom + "_" + $currTo + ".txt"; # Symbol file were exchange rates are currently saved
    # Getting date of last quote for symbol in quote file we already have. If there is no quote file, use min date from parameter file
    if (Test-Path -Path $symbolQuoteFile) { # File exists, need to find last available quote date
        $fc = @(import-csv $symbolQuoteFile -Header "Date","Rate","CurrFrom","CurrTo" | Sort-Object -Property "Date"); $lastDateInFile=""; if ($fc.count -gt 0) {$lastDateInFile = $fc[$fc.count-1].Date;}
        if ($lastDateInFile -eq $null -or $lastDateInFile -eq "") {continue;}
    }
    if ($date -gt $lastDateInFile -and (!$weekendFlag)) {
        "$date,$price,$currFrom,$currTo" | Out-File $quoteIDFileYA -Encoding OEM -append; $newQuotes++; if("$date $time" -gt $lastDateTime) {$lastDateTime = "$date $time"}
        $str=$log + ". Last quote in file: " + $lastDateinFile.PadLeft(10) + ". New quote date: " + $date + " " + $time;
        if($verbose) {$str}; $str | Out-File $logFile -Encoding OEM -append;
    }
    else {$str=$log + ". Local quote file already is up to date."; if($verbose) {$str}; $str | Out-File $logFile -Encoding OEM -append; }
}
$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
$str =(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes received: $newQuotes. Last Quote: $lastDateTime. Duration: $duration`r`n";
$str+="=======================================================================================`r`n";
$logSummary += ". Quotes received: $newQuotes. Last Quote: $lastDateTime. Duration: $duration`r`n";
if($verbose) {$str} else{$logSummary;}; $str | Out-File $logFile -Encoding OEM -append;
